@@ -163,10 +163,10 @@ def convert_to_milnor_matrix(n, basis, p=2, generic='auto'):
         [1 2 0]
     """
     from sage.matrix.constructor import matrix
-    from sage.rings.all import GF
+    from sage.rings.finite_rings.finite_field_constructor import GF
     from .steenrod_algebra import SteenrodAlgebra
     if generic == 'auto':
-        generic = False if p==2 else True
+        generic = p != 2
     if n == 0:
         return matrix(GF(p), 1, 1, [[1]])
     milnor_base = steenrod_algebra_basis(n,'milnor',p, generic=generic)
@@ -337,7 +337,7 @@ def steenrod_algebra_basis(n, basis='milnor', p=2, **kwds):
     except TypeError:
         return ()
 
-    generic = kwds.get("generic", False if p==2 else True)
+    generic = kwds.get("generic", p != 2)
 
     basis_name = get_basis_name(basis, p, generic=generic)
     if basis_name.find('long') >= 0:
@@ -479,7 +479,7 @@ def xi_degrees(n,p=2, reverse=True):
         sage: sage.algebras.steenrod.steenrod_algebra_bases.xi_degrees(400,p=17)
         [307, 18, 1]
     """
-    from sage.rings.all import Integer
+    from sage.rings.integer import Integer
     if n <= 0:
         return []
     N = Integer(n*(p-1) + 1)
@@ -563,7 +563,7 @@ def milnor_basis(n, p=2, **kwds):
         sage: len(milnor_basis(240,7, profile=((),()), truncation_type=0))
         0
     """
-    generic = kwds.get('generic', False if p==2 else True)
+    generic = kwds.get('generic', p != 2)
 
     if n == 0:
         if not generic:
@@ -629,7 +629,7 @@ def milnor_basis(n, p=2, **kwds):
                     # with distinct parts.
                     for sigma in restricted_partitions(deg,
                                                        q_degrees_decrease,
-                                                       no_repeats = True):
+                                                       no_repeats=True):
                         index = 0
                         q_mono = []
                         for q in q_degrees:
@@ -701,7 +701,7 @@ def serre_cartan_basis(n, p=2, bound=1, **kwds):
         sage: serre_cartan_basis(13, 3, bound=3)
         ((1, 3, 0),)
     """
-    generic = kwds.get('generic', False if p==2 else True )
+    generic = kwds.get('generic', p != 2)
 
     if n == 0:
         return ((),)
@@ -713,7 +713,7 @@ def serre_cartan_basis(n, p=2, bound=1, **kwds):
             # This means that 2 last <= n - last, or 3 last <= n.
             result = [(n,)]
             for last in range(bound, 1+n//3):
-                for vec in serre_cartan_basis(n - last, bound = 2*last):
+                for vec in serre_cartan_basis(n - last, bound=2 * last):
                     new = vec + (last,)
                     result.append(new)
         else: # p odd
@@ -1022,14 +1022,14 @@ def atomic_basis_odd(n, basis, p, **kwds):
         elif basis.find('revz') >= 0:
             return (s+t,s)
 
-    generic = kwds.get('generic', False if p==2 else True )
+    generic = kwds.get('generic', p != 2)
     if n == 0:
         if not generic:
             return ((),)
         else:
             return (((), ()),)
 
-    from sage.rings.all import Integer
+    from sage.rings.integer import Integer
     from sage.rings.infinity import Infinity
     from sage.combinat.integer_vector_weighted import WeightedIntegerVectors
     profile = kwds.get("profile", None)
@@ -1057,13 +1057,9 @@ def atomic_basis_odd(n, basis, p, **kwds):
                 # with distinct parts.
                 for sigma in restricted_partitions(deg,
                                                    q_degrees_decrease,
-                                                   no_repeats = True):
-                    index = 0
-                    q_mono = []
-                    for q in q_degrees:
-                        if q in sigma:
-                            q_mono.append(index)
-                        index += 1
+                                                   no_repeats=True):
+                    q_mono = [index for index, q in enumerate(q_degrees)
+                              if q in sigma]
                     # check profile:
                     okay = True
                     if profile is not None and profile != ((), ()):
@@ -1074,7 +1070,7 @@ def atomic_basis_odd(n, basis, p, **kwds):
                                 okay = False
                                 break
 
-                        for ((s,t), exp) in p_mono:
+                        for ((s, t), _) in p_mono:
                             if ((len(profile[0]) > t-1 and profile[0][t-1] <= s)
                                 or (len(profile[0]) <= t-1 and trunc < Infinity)):
                                 okay = False
@@ -1117,7 +1113,7 @@ def steenrod_basis_error_check(dim, p, **kwds):
         sage: steenrod_basis_error_check(80,5) # long time
     """
     from sage.misc.verbose import verbose
-    generic = kwds.get('generic', False if p==2 else True )
+    generic = kwds.get('generic', p != 2)
 
     if not generic:
         bases = ('adem','woody', 'woodz', 'wall', 'arnona', 'arnonc',
