@@ -35,7 +35,7 @@ import copy
 from sage.misc.classcall_metaclass import ClasscallMetaclass
 from sage.misc.inherit_comparison import InheritComparisonClasscallMetaclass
 from sage.misc.flatten import flatten
-from sage.misc.all import prod
+from sage.misc.misc_c import prod
 from sage.structure.unique_representation import UniqueRepresentation
 from sage.structure.parent import Parent
 from sage.structure.element import Element
@@ -44,8 +44,8 @@ from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
 from sage.matrix.matrix_space import MatrixSpace
 from sage.matrix.constructor import matrix
 from sage.modules.free_module_element import zero_vector
-from sage.misc.all import cached_method
-from sage.rings.all import ZZ
+from sage.misc.cachefunc import cached_method
+from sage.rings.integer_ring import ZZ
 from sage.arith.all import factorial
 from sage.rings.integer import Integer
 from sage.combinat.posets.lattices import LatticePoset
@@ -313,7 +313,7 @@ class AlternatingSignMatrix(Element,
             sage: asm = A([[0, 1, 0],[1, -1, 1],[0, 1, 0]])
             sage: asm.inversion_number()
             2
-            sage: P=Permutations(5)
+            sage: P = Permutations(5)
             sage: all(p.number_of_inversions()==AlternatingSignMatrix(p.to_matrix()).inversion_number() for p in P)
             True
         """
@@ -604,7 +604,7 @@ class AlternatingSignMatrix(Element,
             [0 0 1 0]
 
             sage: a0 = a = AlternatingSignMatrices(5).random_element()
-            sage: for i in range(10):
+            sage: for i in range(20):
             ....:     a = a.gyration()
             sage: a == a0
             True
@@ -848,7 +848,7 @@ class AlternatingSignMatrix(Element,
             sage: asm.to_dyck_word()
             Traceback (most recent call last):
             ...
-            TypeError: to_dyck_word() ...argument...
+            TypeError: ...to_dyck_word() ...argument...
             sage: asm.to_dyck_word(algorithm = 'notamethod')
             Traceback (most recent call last):
             ...
@@ -1364,7 +1364,7 @@ class AlternatingSignMatrices(UniqueRepresentation, Parent):
         n = len(comps)
         M = [[0 for _ in range(n)] for _ in range(n)]
 
-        previous_set = set([])
+        previous_set = set()
         for col in range(n-1, -1, -1):
             s = set(comps[col])
             for x in s.difference(previous_set):
@@ -1723,10 +1723,9 @@ class MonotoneTriangles(GelfandTsetlinPatternsTopRow):
             sage: P.is_lattice()
             True
         """
-        # get a list of the elements and switch to a tuple
-        # representation
-        set_ = [tuple(tuple(_) for _ in x) for x in list(self)]
-        return (set_, [(a,b) for a in set_ for b in set_ if _is_a_cover(a,b)])
+        # get a list of the elements and switch to a tuple representation
+        set_ = [tuple(tuple(t) for t in x) for x in list(self)]
+        return (set_, [(a, b) for a in set_ for b in set_ if _is_a_cover(a, b)])
 
     def cover_relations(self):
         r"""
