@@ -24,7 +24,7 @@ from sage.structure.richcmp import richcmp
 #from sage.misc.inherit_comparison import InheritComparisonClasscallMetaclass
 from sage.misc.cachefunc import cached_method
 from sage.misc.misc import powerset
-from sage.functions.other import binomial
+from sage.arith.all import binomial
 from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
 from sage.categories.algebras import Algebras
 from sage.combinat.diagram_algebras import (TemperleyLiebDiagrams, diagram_latex,
@@ -429,7 +429,7 @@ class BlobAlgebra(CombinatorialFreeModule):
         q1 = base_ring(q1)
         q2 = base_ring(q2)
         q3 = base_ring(q3)
-        return super(BlobAlgebra, cls).__classcall__(cls, k, q1, q2, q3, base_ring, prefix)
+        return super().__classcall__(cls, k, q1, q2, q3, base_ring, prefix)
 
     def __init__(self, k, q1, q2, q3, base_ring, prefix):
         r"""
@@ -498,32 +498,32 @@ class BlobAlgebra(CombinatorialFreeModule):
             sage: R.<q,r,s> = ZZ[]
             sage: B2 = algebras.Blob(2, q, r, s)
             sage: latex(B2.an_element())  # indirect doctest
-            2\begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}] 
-            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt] 
-            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {}; 
-            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {}; 
-            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {}; 
-            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {}; 
-            \draw[] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. (G--1); 
-            \draw[] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. (G-2); 
-            \end{tikzpicture} 
-             + 3\begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}] 
-            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt] 
-            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {}; 
-            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {}; 
-            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {}; 
-            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {}; 
-            \draw[blue,very thick] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. node[midway,circle,fill,scale=0.6] {} (G--1); 
-            \draw[] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. (G-2); 
-            \end{tikzpicture} 
-             + 2\begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}] 
-            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt] 
-            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {}; 
-            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {}; 
-            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {}; 
-            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {}; 
-            \draw[blue,very thick] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. node[midway,circle,fill,scale=0.6] {} (G-2); 
-            \draw[] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. (G--1); 
+            2 \begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}]
+            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt]
+            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {};
+            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {};
+            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {};
+            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {};
+            \draw[] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. (G--1);
+            \draw[] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. (G-2);
+            \end{tikzpicture}
+             + 3 \begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}]
+            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt]
+            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {};
+            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {};
+            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {};
+            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {};
+            \draw[blue,very thick] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. node[midway,circle,fill,scale=0.6] {} (G--1);
+            \draw[] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. (G-2);
+            \end{tikzpicture}
+             + 2 \begin{tikzpicture}[scale = 0.5,thick, baseline={(0,-1ex/2)}]
+            \tikzstyle{vertex} = [shape = circle, minimum size = 7pt, inner sep = 1pt]
+            \node[vertex] (G-1) at (0.0, 1) [shape = circle, draw] {};
+            \node[vertex] (G-2) at (1.5, 1) [shape = circle, draw] {};
+            \node[vertex] (G--2) at (1.5, -1) [shape = circle, draw] {};
+            \node[vertex] (G--1) at (0.0, -1) [shape = circle, draw] {};
+            \draw[blue,very thick] (G-1) .. controls +(0.5, -0.5) and +(-0.5, -0.5) .. node[midway,circle,fill,scale=0.6] {} (G-2);
+            \draw[] (G--2) .. controls +(-0.5, 0.5) and +(0.5, 0.5) .. (G--1);
             \end{tikzpicture}
         """
         def edge_options(P):
@@ -532,6 +532,7 @@ class BlobAlgebra(CombinatorialFreeModule):
             if tuple(P) in diagram.marked:
                 return 'blue,very thick'
             return ''
+
         def edge_additions(P):
             if P[1] < P[0]:
                 P = [P[1], P[0]]
@@ -678,4 +679,3 @@ class BlobAlgebra(CombinatorialFreeModule):
             return self.zero()
         diagram = self._indices.element_class(self._indices, ret_lists[0], ret_lists[1])
         return self._from_dict({diagram: coeff}, remove_zeros=False)
-
