@@ -17,30 +17,33 @@ EXAMPLES::
 
 AUTHORS:
 
-- Mike Hansen, Nicolas M. Thiery
+- Mike Hansen, Nicolas M. Thiéry
 - MuPAD-Combinat developers (algorithms and design inspiration)
 - Travis Scrimshaw (2013-02-03): Removed ``CombinatorialClass``
 """
-#*****************************************************************************
+# ****************************************************************************
 #       Copyright (C) 2007 Mike Hansen       <mhansen@gmail.com>
 #                     2009 Nicolas M. Thiery <nthiery at users.sf.net>
 #
 #  Distributed under the terms of the GNU General Public License (GPL)
-#              http://www.gnu.org/licenses/
-#*****************************************************************************
+#              https://www.gnu.org/licenses/
+# ****************************************************************************
+from __future__ import annotations
+from itertools import accumulate
 
 from sage.categories.infinite_enumerated_sets import InfiniteEnumeratedSets
 from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
 from sage.structure.unique_representation import UniqueRepresentation
 from sage.structure.parent import Parent
 from sage.sets.finite_enumerated_set import FiniteEnumeratedSet
-from sage.rings.all import ZZ
+from sage.rings.integer_ring import ZZ
 from .combinat import CombinatorialElement
 from sage.categories.cartesian_product import cartesian_product
 
 from .integer_lists import IntegerListsLex
 from sage.rings.integer import Integer
 from sage.combinat.combinatorial_map import combinatorial_map
+from sage.misc.persist import register_unpickle_override
 
 
 class Composition(CombinatorialElement):
@@ -208,7 +211,7 @@ class Composition(CombinatorialElement):
             self.__dict__ = state[1]
 
     @combinatorial_map(order=2, name='conjugate')
-    def conjugate(self):
+    def conjugate(self) -> Composition:
         r"""
         Return the conjugate of the composition ``self``.
 
@@ -244,20 +247,22 @@ class Composition(CombinatorialElement):
             Compositions of 0
         """
         comp = self
-        if comp == []:
+        if not comp:
             return self
         n = len(comp)
-        coofcp = [sum(comp[:j])-j+1 for j in range(1,n+1)]
+        coofcp = [sigmaj - j for j, sigmaj in enumerate(accumulate(comp))]
 
         cocjg = []
-        for i in range(n-1):
-            cocjg += [i+1 for _ in range(0, (coofcp[n-i-1]-coofcp[n-i-2]))]
+        for i in range(n - 1):
+            ni = n - i
+            cocjg += [i + 1 for _ in range(coofcp[ni - 1] - coofcp[ni - 2])]
         cocjg += [n for j in range(coofcp[0])]
 
-        return self.parent()([cocjg[0]] + [cocjg[i]-cocjg[i-1]+1 for i in range(1,len(cocjg))])
+        return self.parent()([cocjg[0]] + [cocjg[i] - cocjg[i - 1] + 1
+                                           for i in range(1, len(cocjg))])
 
     @combinatorial_map(order=2, name='reversed')
-    def reversed(self):
+    def reversed(self) -> Composition:
         r"""
         Return the reverse composition of ``self``.
 
@@ -272,7 +277,7 @@ class Composition(CombinatorialElement):
         return self.parent()(reversed(self))
 
     @combinatorial_map(order=2, name='complement')
-    def complement(self):
+    def complement(self) -> Composition:
         r"""
         Return the complement of the composition ``self``.
 
@@ -300,7 +305,7 @@ class Composition(CombinatorialElement):
         """
         return self.conjugate().reversed()
 
-    def __add__(self, other):
+    def __add__(self, other) -> Composition:
         """
         Return the concatenation of two compositions.
 
@@ -314,9 +319,9 @@ class Composition(CombinatorialElement):
             sage: Composition([]) + Composition([]) == Composition([])
             True
         """
-        return Compositions()(list(self)+list(other))
+        return Compositions()(list(self) + list(other))
 
-    def size(self):
+    def size(self) -> int:
         """
         Return the size of ``self``, that is the sum of its parts.
 
@@ -328,7 +333,7 @@ class Composition(CombinatorialElement):
         return sum(self)
 
     @staticmethod
-    def sum(compositions):
+    def sum(compositions) -> Composition:
         """
         Return the concatenation of the given compositions.
 
@@ -385,7 +390,7 @@ class Composition(CombinatorialElement):
             sage: Composition([3, 2]).near_concatenation(Composition([]))
             <BLANKLINE>
         """
-        if len(self) == 0 or len(other) == 0:
+        if not self or not other:
             return None
         return Compositions()(list(self)[:-1] + [self[-1] + other[0]] + list(other)[1:])
 
@@ -509,7 +514,7 @@ class Composition(CombinatorialElement):
 
         return (tuple(factors), tuple(signs))
 
-    def join(self, other, check=True):
+    def join(self, other, check=True) -> Composition:
         r"""
         Return the join of ``self`` with a composition ``other`` of the
         same size.
@@ -612,7 +617,7 @@ class Composition(CombinatorialElement):
         if check and (sum(self) != sum(other)):
             raise ValueError("{} is not the same size as {}".format(self, other))
 
-        factors = []
+        factors: list[int] = []
 
         I_iter = iter(self)
         i = 0
@@ -634,11 +639,11 @@ class Composition(CombinatorialElement):
                         i -= j - current_factor_size
                     break
 
-        return Compositions()(factors)
+        return self.parent()(factors)
 
     sup = join
 
-    def meet(self, other, check=True):
+    def meet(self, other, check=True) -> Composition:
         r"""
         Return the meet of ``self`` with a composition ``other`` of the
         same size.
@@ -756,7 +761,7 @@ class Composition(CombinatorialElement):
                         current_part += j - current_factor_size
                     break
 
-        return Compositions()(factors)
+        return self.parent()(factors)
 
     inf = meet
 
@@ -780,7 +785,7 @@ class Composition(CombinatorialElement):
         else:
             return cartesian_product([Compositions(i) for i in self]).map(Composition.sum)
 
-    def is_finer(self, co2):
+    def is_finer(self, co2) -> bool:
         """
         Return ``True`` if the composition ``self`` is finer than the
         composition ``co2``; otherwise, return ``False``.
@@ -798,7 +803,7 @@ class Composition(CombinatorialElement):
         """
         co1 = self
         if sum(co1) != sum(co2):
-            raise ValueError("compositions self (= %s) and co2 (= %s) must be of the same size"%(self, co2))
+            raise ValueError("compositions self (= %s) and co2 (= %s) must be of the same size" % (self, co2))
 
         sum1 = 0
         sum2 = 0
@@ -813,7 +818,7 @@ class Composition(CombinatorialElement):
 
         return True
 
-    def fatten(self, grouping):
+    def fatten(self, grouping) -> Composition:
         r"""
         Return the composition fatter than ``self``, obtained by grouping
         together consecutive parts according to ``grouping``.
@@ -854,12 +859,13 @@ class Composition(CombinatorialElement):
             sage: c.fatten(Composition([3,1,1])).__class__ == c.__class__
             True
         """
-        result = [None] * len(grouping)
+        parent = self.parent()
+        result = [0] * len(grouping)
         j = 0
-        for i in range(len(grouping)):
-            result[i] = sum(self[j:j+grouping[i]])
-            j += grouping[i]
-        return Compositions()(result)
+        for i, gi in enumerate(grouping):
+            result[i] = sum(self[j:j + gi])
+            j += gi
+        return parent(result)
 
     def fatter(self):
         """
@@ -887,7 +893,7 @@ class Composition(CombinatorialElement):
         """
         return Compositions(len(self)).map(self.fatten)
 
-    def refinement_splitting(self, J):
+    def refinement_splitting(self, J) -> list[Composition]:
         r"""
         Return the refinement splitting of ``self`` according to ``J``.
 
@@ -922,8 +928,8 @@ class Composition(CombinatorialElement):
         """
         I = self
         if sum(I) != sum(J):
-            #Error: compositions are not of the same size
-            raise ValueError("compositions self (= %s) and J (= %s) must be of the same size"%(I, J))
+            # Error: compositions are not of the same size
+            raise ValueError("compositions self (= %s) and J (= %s) must be of the same size" % (I, J))
         sum1 = 0
         sum2 = 0
         i1 = -1
@@ -936,7 +942,7 @@ class Composition(CombinatorialElement):
                 new_comp.append(I[i1])
                 sum1 += new_comp[-1]
             if sum1 > sum2:
-                raise ValueError("composition J (= %s) does not refine self (= %s)"%(I, J))
+                raise ValueError("composition J (= %s) does not refine self (= %s)" % (I, J))
             decomp.append(Compositions()(new_comp))
         return decomp
 
@@ -964,9 +970,9 @@ class Composition(CombinatorialElement):
             ...
             ValueError: composition J (= [2, 1]) does not refine self (= [1, 2])
         """
-        return Compositions()([len(_) for _ in self.refinement_splitting(J)])
+        return Compositions()([len(p) for p in self.refinement_splitting(J)])
 
-    def major_index(self):
+    def major_index(self) -> int:
         """
         Return the major index of ``self``. The major index is
         defined as the sum of the descents.
@@ -976,35 +982,41 @@ class Composition(CombinatorialElement):
             sage: Composition([1, 1, 3, 1, 2, 1, 3]).major_index()
             31
         """
-        co = self
-        lv = len(co)
+        lv = len(self)
         if lv == 1:
             return 0
-        else:
-            return sum([(lv-(i+1))*co[i] for i in range(lv)])
+        return sum([(lv - (i + 1)) * ci
+                    for i, ci in enumerate(self)])
 
-    def to_code(self):
+    def to_code(self) -> list:
         r"""
-        Return the code of the composition ``self``. The code of a composition
-        `I` is a list of length `\mathrm{size}(I)` of 1s and 0s such that
-        there is a 1 wherever a new part starts. (Exceptional case: When the
+        Return the code of the composition ``self``.
+
+        The code of a composition `I` is a list of length
+        `\mathrm{size}(I)` of 1s and 0s such that there is a 1
+        wherever a new part starts. (Exceptional case: When the
         composition is empty, the code is ``[0]``.)
 
         EXAMPLES::
 
             sage: Composition([4,1,2,3,5]).to_code()
             [1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0]
+
+        TESTS::
+
+            sage: Composition([]).to_code()
+            [0]
         """
-        if self == []:
+        if not self:
             return [0]
 
         code = []
         for i in self:
-            code += [1] + [0]*(i-1)
+            code += [1] + [0] * (i - 1)
 
         return code
 
-    def partial_sums(self, final=True):
+    def partial_sums(self, final=True) -> list:
         r"""
         The partial sums of the sequence defined by the entries of the
         composition.
@@ -1090,7 +1102,7 @@ class Composition(CombinatorialElement):
         from sage.sets.set import Set
         return Set(self.partial_sums(final=final))
 
-    def descents(self, final_descent=False):
+    def descents(self, final_descent=False) -> list:
         r"""
         This gives one fewer than the partial sums of the composition.
 
@@ -1122,10 +1134,11 @@ class Composition(CombinatorialElement):
         """
         return [i - 1 for i in self.partial_sums(final=final_descent)]
 
-    def peaks(self):
+    def peaks(self) -> list:
         """
-        Return a list of the peaks of the composition ``self``. The
-        peaks of a composition are the descents which do not
+        Return a list of the peaks of the composition ``self``.
+
+        The peaks of a composition are the descents which do not
         immediately follow another descent.
 
         EXAMPLES::
@@ -1133,9 +1146,9 @@ class Composition(CombinatorialElement):
             sage: Composition([1, 1, 3, 1, 2, 1, 3]).peaks()
             [4, 7]
         """
-        descents = dict((d-1,True) for d in self.to_subset(final=True))
-        return [i+1 for i in range(len(self))
-                if i not in descents and i+1 in descents]
+        descents = set(d - 1 for d in self.to_subset(final=True))
+        return [i + 1 for i in range(len(self))
+                if i not in descents and i + 1 in descents]
 
     @combinatorial_map(name='to partition')
     def to_partition(self):
@@ -1157,13 +1170,15 @@ class Composition(CombinatorialElement):
 
     def to_skew_partition(self, overlap=1):
         """
-        Return the skew partition obtained from ``self``. This is a
-        skew partition whose rows have the entries of ``self`` as their
-        length, taken in reverse order (so the first entry of ``self``
-        is the length of the lowermost row, etc.). The parameter
-        ``overlap`` indicates the number of cells on each row that are
-        directly below cells of the previous row. When it is set to `1`
-        (its default value), the result is the ribbon shape of ``self``.
+        Return the skew partition obtained from ``self``.
+
+        This is a skew partition whose rows have the entries of
+        ``self`` as their length, taken in reverse order (so the first
+        entry of ``self`` is the length of the lowermost row,
+        etc.). The parameter ``overlap`` indicates the number of cells
+        on each row that are directly below cells of the previous
+        row. When it is set to `1` (its default value), the result is
+        the ribbon shape of ``self``.
 
         EXAMPLES::
 
@@ -1181,7 +1196,7 @@ class Composition(CombinatorialElement):
         from sage.combinat.skew_partition import SkewPartition
         outer = []
         inner = []
-        sum_outer = -1*overlap
+        sum_outer = -overlap
 
         for k in self[:-1]:
             outer.append(k + sum_outer + overlap)
@@ -1191,11 +1206,11 @@ class Composition(CombinatorialElement):
         if self:
             outer.append(self[-1] + sum_outer + overlap)
         else:
-            return SkewPartition([[],[]])
+            return SkewPartition([[], []])
 
         return SkewPartition(
-            [ [x for x in reversed(outer) if x != 0],
-              [x for x in reversed(inner) if x != 0] ])
+            [[x for x in reversed(outer) if x != 0],
+             [x for x in reversed(inner) if x != 0]])
 
     def shuffle_product(self, other, overlap=False):
         r"""
@@ -1271,7 +1286,7 @@ class Composition(CombinatorialElement):
             from sage.combinat.words.shuffle_product import ShuffleProduct_w1w2
             return ShuffleProduct_w1w2(self, other)
 
-    def wll_gt(self, co2):
+    def wll_gt(self, co2) -> bool:
         """
         Return ``True`` if the composition ``self`` is greater than the
         composition ``co2`` with respect to the wll-ordering; otherwise,
@@ -1337,6 +1352,7 @@ class Composition(CombinatorialElement):
 
 ##############################################################
 
+
 class Compositions(UniqueRepresentation, Parent):
     r"""
     Set of integer compositions.
@@ -1580,18 +1596,20 @@ class Compositions(UniqueRepresentation, Parent):
             True
         """
         if n is None:
-            if len(kwargs) != 0:
+            if kwargs:
                 raise ValueError("Incorrect number of arguments")
             return Compositions_all()
         else:
-            if len(kwargs) == 0:
-                if isinstance(n, (int,Integer)):
+            if not kwargs:
+                if isinstance(n, (int, Integer)):
                     return Compositions_n(n)
                 else:
                     raise ValueError("n must be an integer")
             else:
                 # FIXME: should inherit from IntegerListLex, and implement repr, or _name as a lazy attribute
-                kwargs['name'] = "Compositions of the integer %s satisfying constraints %s"%(n, ", ".join( ["%s=%s"%(key, kwargs[key]) for key in sorted(kwargs)] ))
+                txt = "Compositions of the integer %s satisfying constraints %s"
+                kwargs['name'] = txt % (n, ", ".join(f"{key}={kwargs[key]}"
+                                                     for key in sorted(kwargs)))
                 kwargs['element_class'] = Composition
                 if 'min_part' not in kwargs:
                     kwargs['min_part'] = 1
@@ -1634,7 +1652,7 @@ class Compositions(UniqueRepresentation, Parent):
 
     Element = Composition
 
-    def _element_constructor_(self, lst):
+    def _element_constructor_(self, lst) -> Composition:
         """
         Construct an element with ``self`` as parent.
 
@@ -1648,10 +1666,10 @@ class Compositions(UniqueRepresentation, Parent):
             lst = list(lst)
         elt = self.element_class(self, lst)
         if elt not in self:
-            raise ValueError("%s not in %s"%(elt, self))
+            raise ValueError("%s not in %s" % (elt, self))
         return elt
 
-    def __contains__(self, x):
+    def __contains__(self, x) -> bool:
         """
         TESTS::
 
@@ -1676,7 +1694,7 @@ class Compositions(UniqueRepresentation, Parent):
         else:
             return False
 
-    def from_descents(self, descents, nps=None):
+    def from_descents(self, descents, nps=None) -> Composition:
         """
         Return a composition from the list of descents.
 
@@ -1702,12 +1720,12 @@ class Compositions(UniqueRepresentation, Parent):
             sage: Compositions().from_descents([1,0,4,8,11])
             [1, 1, 3, 4, 3]
         """
-        d = [x+1 for x in sorted(descents)]
+        d = [x + 1 for x in sorted(descents)]
         if nps is None:
             nps = d.pop()
         return self.from_subset(d, nps)
 
-    def from_subset(self, S, n):
+    def from_subset(self, S, n) -> Composition:
         r"""
         The composition of `n` corresponding to the subset ``S`` of
         `\{1, 2, \ldots, n-1\}` under the bijection that maps the composition
@@ -1742,24 +1760,24 @@ class Compositions(UniqueRepresentation, Parent):
         """
         d = sorted(S)
 
-        if d == []:
+        if not d:
             if n == 0:
                 return self.element_class(self, [])
             else:
                 return self.element_class(self, [n])
 
         if n <= d[-1]:
-            raise ValueError("S (=%s) is not a subset of {1, ..., %s}" % (d,n-1))
+            raise ValueError("S (=%s) is not a subset of {1, ..., %s}" % (d, n - 1))
         else:
             d.append(n)
 
         co = [d[0]]
-        for i in range(len(d)-1):
-            co.append(d[i+1]-d[i])
+        for i in range(len(d) - 1):
+            co.append(d[i + 1] - d[i])
 
         return self.element_class(self, co)
 
-    def from_code(self, code):
+    def from_code(self, code) -> Composition:
         r"""
         Return the composition from its code. The code of a composition
         `I` is a list of length `\mathrm{size}(I)` consisting of 1s and
@@ -1779,10 +1797,12 @@ class Compositions(UniqueRepresentation, Parent):
             [3, 1, 2, 3, 5]
         """
         if code == [0]:
-            return []
+            return self.element_class(self, [])
+
+        L = [x for x in range(len(code)) if code[x] == 1]  # the positions of the letter 1
+        c = [L[i] - L[i - 1] for i in range(1, len(L))] + [len(code) - L[-1]]
+        return self.element_class(self, c)
 
-        L = [x for x in range(len(code)) if code[x]==1] #the positions of the letter 1
-        return self.element_class(self, [L[i]-L[i-1] for i in range(1, len(L))] + [len(code)-L[-1]])
 
 # Allows to unpickle old constrained Compositions_constraints objects.
 class Compositions_constraints(IntegerListsLex):
@@ -1802,11 +1822,12 @@ class Compositions_constraints(IntegerListsLex):
         """
         n = data['n']
         self.__class__ = IntegerListsLex
-        constraints = {'min_part' : 1,
-                       'element_class' : Composition}
+        constraints = {'min_part': 1,
+                       'element_class': Composition}
         constraints.update(data['constraints'])
         self.__init__(n, **constraints)
 
+
 class Compositions_all(Compositions):
     """
     Class of all compositions.
@@ -1822,7 +1843,7 @@ class Compositions_all(Compositions):
         """
         Compositions.__init__(self, True)
 
-    def _repr_(self):
+    def _repr_(self) -> str:
         """
         Return a string representation of ``self``.
 
@@ -1866,6 +1887,7 @@ class Compositions_all(Compositions):
                 yield self.element_class(self, list(c))
             n += 1
 
+
 class Compositions_n(Compositions):
     """
     Class of compositions of a fixed `n`.
@@ -1885,7 +1907,7 @@ class Compositions_n(Compositions):
             sage: C is C3
             True
         """
-        return super(Compositions_n, cls).__classcall__(cls, Integer(n))
+        return super().__classcall__(cls, Integer(n))
 
     def __init__(self, n):
         """
@@ -1899,7 +1921,7 @@ class Compositions_n(Compositions):
         self.n = n
         Compositions.__init__(self, False)
 
-    def _repr_(self):
+    def _repr_(self) -> str:
         """
         Return a string representation of ``self``.
 
@@ -1908,9 +1930,9 @@ class Compositions_n(Compositions):
             sage: repr(Compositions(3))
             'Compositions of 3'
         """
-        return "Compositions of %s"%self.n
+        return "Compositions of %s" % self.n
 
-    def __contains__(self, x):
+    def __contains__(self, x) -> bool:
         """
         TESTS::
 
@@ -1925,7 +1947,7 @@ class Compositions_n(Compositions):
         """
         return x in Compositions() and sum(x) == self.n
 
-    def cardinality(self):
+    def cardinality(self) -> Integer:
         """
         Return the number of compositions of `n`.
 
@@ -1937,13 +1959,13 @@ class Compositions_n(Compositions):
             1
         """
         if self.n >= 1:
-            return ZZ(2) ** (self.n-1)
+            return ZZ(2)**(self.n - 1)
         elif self.n == 0:
-            return ZZ(1)
+            return ZZ.one()
         else:
-            return ZZ(0)
+            return ZZ.zero()
 
-    def random_element(self):
+    def random_element(self) -> Composition:
         r"""
         Return a random ``Composition`` with uniform probability.
 
@@ -1968,7 +1990,7 @@ class Compositions_n(Compositions):
 
         if self.n == 0:
             return Compositions()([])
-        return Compositions().from_code([1] + [choice([0,1]) for _ in range(self.n - 1)])
+        return Compositions().from_code([1] + [choice([0, 1]) for _ in range(self.n - 1)])
 
     def __iter__(self):
         """
@@ -1984,6 +2006,7 @@ class Compositions_n(Compositions):
         for c in composition_iterator_fast(self.n):
             yield self.element_class(self, c)
 
+
 def composition_iterator_fast(n):
     """
     Iterator over compositions of ``n`` yielded as lists.
@@ -1994,7 +2017,7 @@ def composition_iterator_fast(n):
         sage: L = list(composition_iterator_fast(4)); L
         [[1, 1, 1, 1], [1, 1, 2], [1, 2, 1], [1, 3], [2, 1, 1], [2, 2], [3, 1], [4]]
         sage: type(L[0])
-        <... 'list'>
+        <class 'list'>
     """
     # Special cases
     if n < 0:
@@ -2003,7 +2026,7 @@ def composition_iterator_fast(n):
         yield []
         return
 
-    s = Integer(0) # Current sum
+    s = Integer(0)  # Current sum
     cur = [Integer(0)]
     while cur:
         cur[-1] += 1
@@ -2016,6 +2039,5 @@ def composition_iterator_fast(n):
         else:
             cur.append(Integer(0))
 
-from sage.misc.persist import register_unpickle_override
-register_unpickle_override('sage.combinat.composition', 'Composition_class', Composition)
 
+register_unpickle_override('sage.combinat.composition', 'Composition_class', Composition)
