@@ -148,7 +148,8 @@ from sage.structure.unique_representation import UniqueRepresentation
 from sage.categories.infinite_enumerated_sets import InfiniteEnumeratedSets
 from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
 
-from sage.rings.all import ZZ, QQ
+from sage.rings.integer_ring import ZZ
+from sage.rings.rational_field import QQ
 from sage.sets.set import Set
 from sage.graphs.digraph import DiGraph
 from sage.matrix.matrix_space import MatrixSpace
@@ -180,9 +181,9 @@ class SkewPartition(CombinatorialElement):
             sage: skp.outer()
             [3, 2, 1]
         """
-        skp = [_Partitions(_) for _ in skp]
+        skp = [_Partitions(p) for p in skp]
         if skp not in SkewPartitions():
-            raise ValueError("invalid skew partition: %s"%skp)
+            raise ValueError("invalid skew partition: %s" % skp)
         return SkewPartitions()(skp)
 
     def __init__(self, parent, skp):
@@ -218,7 +219,7 @@ class SkewPartition(CombinatorialElement):
             sage: print(SkewPartition([[3,2,1],[2,1]])._repr_quotient())
             [3, 2, 1] / [2, 1]
         """
-        return "%s / %s"%(self[0], self[1])
+        return "%s / %s" % (self[0], self[1])
 
     def _repr_lists(self):
         """
@@ -229,7 +230,7 @@ class SkewPartition(CombinatorialElement):
             sage: print(SkewPartition([[3,2,1],[2,1]])._repr_lists())
             [[3, 2, 1], [2, 1]]
         """
-        return repr([list(_) for _ in self])
+        return repr([list(r) for r in self])
 
     def _latex_(self):
         r"""
@@ -635,13 +636,14 @@ class SkewPartition(CombinatorialElement):
 
     def is_ribbon(self):
         r"""
-        Return ``True`` if and only if ``self`` is a ribbon, that is,
-        if it has exactly one cell in each of `q` consecutive
-        diagonals for some nonnegative integer `q`.
+        Return ``True`` if and only if ``self`` is a ribbon.
+
+        This means that if it has exactly one cell in each of `q`
+        consecutive diagonals for some nonnegative integer `q`.
 
         EXAMPLES::
 
-            sage: P=SkewPartition([[4,4,3,3],[3,2,2]])
+            sage: P = SkewPartition([[4,4,3,3],[3,2,2]])
             sage: P.pp()
                *
               **
@@ -650,7 +652,7 @@ class SkewPartition(CombinatorialElement):
             sage: P.is_ribbon()
             True
 
-            sage: P=SkewPartition([[4,3,3],[1,1]])
+            sage: P = SkewPartition([[4,3,3],[1,1]])
             sage: P.pp()
              ***
              **
@@ -658,7 +660,7 @@ class SkewPartition(CombinatorialElement):
             sage: P.is_ribbon()
             False
 
-            sage: P=SkewPartition([[4,4,3,2],[3,2,2]])
+            sage: P = SkewPartition([[4,4,3,2],[3,2,2]])
             sage: P.pp()
                *
               **
@@ -667,7 +669,7 @@ class SkewPartition(CombinatorialElement):
             sage: P.is_ribbon()
             False
 
-            sage: P=SkewPartition([[4,4,3,3],[4,2,2,1]])
+            sage: P = SkewPartition([[4,4,3,3],[4,2,2,1]])
             sage: P.pp()
             <BLANKLINE>
               **
@@ -676,7 +678,7 @@ class SkewPartition(CombinatorialElement):
             sage: P.is_ribbon()
             True
 
-            sage: P=SkewPartition([[4,4,3,3],[4,2,2]])
+            sage: P = SkewPartition([[4,4,3,3],[4,2,2]])
             sage: P.pp()
             <BLANKLINE>
               **
@@ -1004,9 +1006,9 @@ class SkewPartition(CombinatorialElement):
             sage: s.to_list()
             [[4, 3, 1], [2]]
             sage: type(s.to_list())
-            <... 'list'>
+            <class 'list'>
         """
-        return [list(_) for _ in list(self)]
+        return [list(r) for r in list(self)]
 
     def to_dag(self, format="string"):
         """
@@ -1027,17 +1029,17 @@ class SkewPartition(CombinatorialElement):
         EXAMPLES::
 
             sage: dag = SkewPartition([[3, 3, 1], [1, 1]]).to_dag()
-            sage: dag.edges()
+            sage: dag.edges(sort=True)
             [('0,1', '0,2', None),
             ('0,1', '1,1', None),
             ('0,2', '1,2', None),
             ('1,1', '1,2', None)]
-            sage: dag.vertices()
+            sage: dag.vertices(sort=True)
             ['0,1', '0,2', '1,1', '1,2', '2,0']
             sage: dag = SkewPartition([[3, 2, 1], [1, 1]]).to_dag(format="tuple")
-            sage: dag.edges()
+            sage: dag.edges(sort=True)
             [((0, 1), (0, 2), None), ((0, 1), (1, 1), None)]
-            sage: dag.vertices()
+            sage: dag.vertices(sort=True)
             [(0, 1), (0, 2), (1, 1), (2, 0)]
         """
         outer = list(self.outer())
@@ -1503,9 +1505,9 @@ class SkewPartitions(UniqueRepresentation, Parent):
         if not all(i>0 for i in rowL) or not all(i>0 for i in colL):
             raise ValueError("row and column length must be positive")
         if rowL == []:
-            return self.element_class(self, [[],[]])
+            return self.element_class(self, [[], []])
         colL_new = colL[:]
-        resIn  = []
+        resIn = []
         resOut = []
         inPOld = len(colL)
         for row in rowL:
@@ -1516,7 +1518,7 @@ class SkewPartitions(UniqueRepresentation, Parent):
             resIn.append(inP)
             resOut.append(len(colL_new))
             for iCol in range(inP, len(colL_new)):
-                colL_new[iCol] -= 1;
+                colL_new[iCol] -= 1
                 if colL_new[iCol] < 0:
                     raise ValueError("Incompatible row and column length : %s and %s"%(rowL, colL))
             while colL_new and colL_new[-1] == 0:
@@ -1574,6 +1576,7 @@ class SkewPartitions_all(SkewPartitions):
                 yield self.element_class(self, p)
             n += 1
 
+
 class SkewPartitions_n(SkewPartitions):
     """
     The set of skew partitions of ``n`` with overlap at least
@@ -1611,7 +1614,7 @@ class SkewPartitions_n(SkewPartitions):
         """
         if overlap == 'connected':
             overlap = 1
-        return super(cls, SkewPartitions_n).__classcall__(cls, n, overlap)
+        return super().__classcall__(cls, n, overlap)
 
     def __init__(self, n, overlap):
         """
@@ -1689,9 +1692,9 @@ class SkewPartitions_n(SkewPartitions):
             sage: SkewPartitions(3, overlap=1)
             Skew partitions of 3 with a minimum overlap of 1
         """
-        string = "Skew partitions of %s"%self.n
+        string = "Skew partitions of %s" % self.n
         if self.overlap:
-            string += " with a minimum overlap of %s"%self.overlap
+            string += " with a minimum overlap of %s" % self.overlap
         return string
 
     def _count_slide(self, co, overlap=0):
@@ -1713,13 +1716,10 @@ class SkewPartitions_n(SkewPartitions):
             sage: [ sp for sp in s if sp.row_lengths() == [2,1] ]
             [[2, 1] / []]
         """
-        nn = len(co)
         result = 1
-        for i in range(nn-1):
-            comb    = min(co[i], co[i+1])
-            comb   += 1 - overlap
-            result *= comb
-
+        shift = 1 - overlap
+        for i in range(len(co) - 1):
+            result *= min(co[i], co[i + 1]) + shift
         return result
 
     def cardinality(self):
@@ -1825,7 +1825,7 @@ class SkewPartitions_rowlengths(SkewPartitions):
         co = Compositions()(co)
         if overlap == 'connected':
             overlap = 1
-        return super(SkewPartitions_rowlengths, cls).__classcall__(cls, co, overlap)
+        return super().__classcall__(cls, co, overlap)
 
     def __init__(self, co, overlap):
         """
@@ -1864,7 +1864,7 @@ class SkewPartitions_rowlengths(SkewPartitions):
             sage: SkewPartitions(row_lengths=[2,1])
             Skew partitions with row lengths [2, 1]
         """
-        return "Skew partitions with row lengths %s"%self.co
+        return "Skew partitions with row lengths %s" % self.co
 
     def _from_row_lengths_aux(self, sskp, ck_1, ck, overlap=0):
         """
