@@ -77,16 +77,20 @@ As you see, it was necessary to insert two new rays (to prevent "upper" and
 
 import operator
 
-from sage.categories.all import Hom
+from sage.categories.homset import Hom
 from sage.geometry.cone import Cone
 from sage.geometry.fan import Fan, is_Fan
-from sage.matrix.all import identity_matrix, matrix
+from sage.matrix.constructor import matrix
+from sage.matrix.special import identity_matrix
 from sage.structure.element import is_Matrix
 from sage.misc.cachefunc import cached_method
-from sage.misc.all import latex, prod, walltime
+from sage.misc.latex import latex
+from sage.misc.misc import walltime
+from sage.misc.misc_c import prod
 from sage.modules.free_module_morphism import (FreeModuleMorphism,
                                                is_FreeModuleMorphism)
-from sage.rings.all import Infinity, ZZ
+from sage.rings.infinity import Infinity
+from sage.rings.integer_ring import ZZ
 from sage.rings.infinity import is_Infinite
 from functools import reduce
 
@@ -283,7 +287,7 @@ class FanMorphism(FreeModuleMorphism):
         else:
             raise TypeError("morphism must be either a FreeModuleMorphism "
                             "or a matrix!\nGot: %s" % morphism)
-        super(FanMorphism, self).__init__(parent, A)
+        super().__init__(parent, A)
         self._domain_fan = domain_fan
         self._image_cone = dict()
         self._preimage_cones = dict()
@@ -314,16 +318,15 @@ class FanMorphism(FreeModuleMorphism):
         EXAMPLES::
 
             sage: A2 = toric_varieties.A2()
-            sage: P3 = toric_varieties.P(3)
+            sage: P3 = toric_varieties.P(3)                                     # optional - palp
             sage: m = matrix([(2,0,0), (1,1,0)])
-            sage: phi = A2.hom(m, P3).fan_morphism()
-            sage: phi
+            sage: phi = A2.hom(m, P3).fan_morphism(); phi                       # optional - palp
             Fan morphism defined by the matrix
             [2 0 0]
             [1 1 0]
             Domain fan: Rational polyhedral fan in 2-d lattice N
             Codomain fan: Rational polyhedral fan in 3-d lattice N
-            sage: prod(phi.factor()) # indirect test
+            sage: prod(phi.factor())  # indirect test                           # optional - palp
             Fan morphism defined by the matrix
             [2 0 0]
             [1 1 0]
@@ -525,13 +528,13 @@ class FanMorphism(FreeModuleMorphism):
 
         TESTS::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
-            sage: phi._ray_index_map()
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
+            sage: phi._ray_index_map()                                          # optional - palp
             (-1, 1, -1, 0)
-            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)
-            sage: xi._ray_index_map()
+            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)            # optional - palp
+            sage: xi._ray_index_map()                                           # optional - palp
             Traceback (most recent call last):
             ...
             ValueError: ray #1 is mapped into a 2-d cone!
@@ -1035,16 +1038,16 @@ class FanMorphism(FreeModuleMorphism):
 
         EXAMPLES::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
-            sage: phi.index()
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
+            sage: phi.index()                                                   # optional - palp
             1
-            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)
-            sage: psi.index()
+            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)        # optional - palp
+            sage: psi.index()                                                   # optional - palp
             2
-            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)
-            sage: xi.index()
+            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)            # optional - palp
+            sage: xi.index()                                                    # optional - palp
             +Infinity
 
         Infinite index in the last example indicates that the image has positive
@@ -1054,13 +1057,13 @@ class FanMorphism(FreeModuleMorphism):
             N( 1),
             N(-1)
             in 1-d lattice N
-            sage: Sigma.rays()
+            sage: Sigma.rays()                                                  # optional - palp
             N( 1,  1),
             N( 0,  1),
             N(-1, -1),
             N( 1,  0)
             in 2-d lattice N
-            sage: xi.factor()[0].domain_fan().rays()
+            sage: xi.factor()[0].domain_fan().rays()                            # optional - palp
             N(-1, 0),
             N( 1, 0)
             in Sublattice <N(1, 0)>
@@ -1076,25 +1079,25 @@ class FanMorphism(FreeModuleMorphism):
         1-d cones are ``None``, except for one which is infinite, and all
         indices over 2-d cones are ``None``, except for one which is 1::
 
-            sage: [xi.index(cone) for cone in Sigma(1)]
+            sage: [xi.index(cone) for cone in Sigma(1)]                         # optional - palp
             [None, None, None, +Infinity]
-            sage: [xi.index(cone) for cone in Sigma(2)]
+            sage: [xi.index(cone) for cone in Sigma(2)]                         # optional - palp
             [None, 1, None, None]
 
         TESTS::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.Cube_nonpolyhedral().fan()
             sage: m = matrix([[2,6,10], [7,11,13]])
-            sage: zeta = FanMorphism(m, Sigma, Sigma_p, subdivide=True)
-            sage: [zeta.index(cone) for cone in flatten(Sigma_p.cones())]
+            sage: zeta = FanMorphism(m, Sigma, Sigma_p, subdivide=True)         # optional - palp
+            sage: [zeta.index(cone) for cone in flatten(Sigma_p.cones())]       # optional - palp
             [+Infinity, None, None, None, None, None, None, None, None, None,
              4, 4, None, 4, None, None, 2, None, 4, None, 4, 1, 1, 1, 1, 1, 1]
-            sage: zeta = prod(zeta.factor()[1:])
-            sage: Sigma_p = zeta.codomain_fan()
-            sage: [zeta.index(cone) for cone in flatten(Sigma_p.cones())]
+            sage: zeta = prod(zeta.factor()[1:])                                # optional - palp
+            sage: Sigma_p = zeta.codomain_fan()                                 # optional - palp
+            sage: [zeta.index(cone) for cone in flatten(Sigma_p.cones())]       # optional - palp
             [4, 4, 4, 1, 4, 4, 4, 1, 1, 1, 1, 1, 1]
-            sage: zeta.index() == zeta.index(Sigma_p(0)[0])
+            sage: zeta.index() == zeta.index(Sigma_p(0)[0])                     # optional - palp
             True
         """
         if cone is None:
@@ -1125,14 +1128,14 @@ class FanMorphism(FreeModuleMorphism):
 
         EXAMPLES::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
-            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)
-            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)
-            sage: phi.index(), psi.index(), xi.index()
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
+            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)        # optional - palp
+            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)            # optional - palp
+            sage: phi.index(), psi.index(), xi.index()                          # optional - palp
             (1, 2, +Infinity)
-            sage: phi.is_birational(), psi.is_birational(), xi.is_birational()
+            sage: phi.is_birational(), psi.is_birational(), xi.is_birational()  # optional - palp
             (True, False, False)
         """
         return self.index() == 1
@@ -1173,26 +1176,26 @@ class FanMorphism(FreeModuleMorphism):
         We consider several maps between fans of a del Pezzo surface and the
         projective line::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
-            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)
-            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)
-            sage: phi.is_bundle()
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
+            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)        # optional - palp
+            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)            # optional - palp
+            sage: phi.is_bundle()                                               # optional - palp
             True
-            sage: phi.is_fibration()
+            sage: phi.is_fibration()                                            # optional - palp
             True
-            sage: phi.index()
+            sage: phi.index()                                                   # optional - palp
             1
-            sage: psi.is_bundle()
+            sage: psi.is_bundle()                                               # optional - palp
             False
-            sage: psi.is_fibration()
+            sage: psi.is_fibration()                                            # optional - palp
             True
-            sage: psi.index()
+            sage: psi.index()                                                   # optional - palp
             2
-            sage: xi.is_fibration()
+            sage: xi.is_fibration()                                             # optional - palp
             False
-            sage: xi.index()
+            sage: xi.index()                                                    # optional - palp
             +Infinity
 
         The first of these maps induces not only a fibration, but a fiber
@@ -1248,7 +1251,7 @@ class FanMorphism(FreeModuleMorphism):
         the linear map of vector spaces `\phi_\RR` induces a bijection
         between `\sigma` and `\sigma'`, and, in addition, `\phi` is
         :meth:`dominant <is_dominant>` (that is, `\phi_\RR: N_\RR \to
-        N'_\RR$ is surjective).
+        N'_\RR` is surjective).
 
         If a fan morphism `\phi: \Sigma \to \Sigma'` is a fibration, then the
         associated morphism between toric varieties `\tilde{\phi}: X_\Sigma \to
@@ -1265,26 +1268,26 @@ class FanMorphism(FreeModuleMorphism):
         We consider several maps between fans of a del Pezzo surface and the
         projective line::
 
-            sage: Sigma = toric_varieties.dP8().fan()
+            sage: Sigma = toric_varieties.dP8().fan()                           # optional - palp
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
-            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)
-            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)
-            sage: phi.is_bundle()
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
+            sage: psi = FanMorphism(matrix([[2], [-2]]), Sigma, Sigma_p)        # optional - palp
+            sage: xi = FanMorphism(matrix([[1, 0]]), Sigma_p, Sigma)            # optional - palp
+            sage: phi.is_bundle()                                               # optional - palp
             True
-            sage: phi.is_fibration()
+            sage: phi.is_fibration()                                            # optional - palp
             True
-            sage: phi.index()
+            sage: phi.index()                                                   # optional - palp
             1
-            sage: psi.is_bundle()
+            sage: psi.is_bundle()                                               # optional - palp
             False
-            sage: psi.is_fibration()
+            sage: psi.is_fibration()                                            # optional - palp
             True
-            sage: psi.index()
+            sage: psi.index()                                                   # optional - palp
             2
-            sage: xi.is_fibration()
+            sage: xi.is_fibration()                                             # optional - palp
             False
-            sage: xi.index()
+            sage: xi.index()                                                    # optional - palp
             +Infinity
 
         The first of these maps induces not only a fibration, but a fiber
@@ -1379,9 +1382,9 @@ class FanMorphism(FreeModuleMorphism):
 
         We also embed the affine plane into the projective one::
 
-            sage: P2 = toric_varieties.P(2).fan()
+            sage: P2 = toric_varieties.P(2).fan()                               # optional - palp
             sage: m = identity_matrix(2)
-            sage: FanMorphism(m, A2, P2).is_injective()
+            sage: FanMorphism(m, A2, P2).is_injective()                         # optional - palp
             True
         """
         if self.matrix().index_in_saturation() != 1:
@@ -1407,7 +1410,7 @@ class FanMorphism(FreeModuleMorphism):
         `\sigma' \in \Sigma'` there is at least one preimage cone `\sigma \in
         \Sigma` such that the relative interior of `\sigma` is mapped to the
         relative interior of `\sigma'` and, in addition,
-        `\phi_\RR: N_\RR \to N'_\RR$ is surjective.
+        `\phi_\RR: N_\RR \to N'_\RR` is surjective.
 
         If a fan morphism `\phi: \Sigma \to \Sigma'` is surjective, then the
         associated morphism between toric varieties `\tilde{\phi}: X_\Sigma \to
@@ -1462,8 +1465,8 @@ class FanMorphism(FreeModuleMorphism):
         r"""
         Return whether the fan morphism is dominant.
 
-        A fan morphism $\phi$ is dominant if it is surjective as a map
-        of vector spaces. That is, $\phi_\RR: N_\RR \to N'_\RR$ is
+        A fan morphism `\phi` is dominant if it is surjective as a map
+        of vector spaces. That is, `\phi_\RR: N_\RR \to N'_\RR` is
         surjective.
 
         If the domain fan is :meth:`complete
@@ -1563,12 +1566,11 @@ class FanMorphism(FreeModuleMorphism):
             sage: N2 = ToricLattice(2)
             sage: Hom21 = Hom(N2, N1)
             sage: pr = Hom21([N1.0,0])
-            sage: P1xP1 = toric_varieties.P1xP1()
-            sage: f = FanMorphism(pr, P1xP1.fan())
-            sage: c = f.image_cone(Cone([(1,0), (0,1)]))
-            sage: c
+            sage: P1xP1 = toric_varieties.P1xP1()                               # optional - palp
+            sage: f = FanMorphism(pr, P1xP1.fan())                              # optional - palp
+            sage: c = f.image_cone(Cone([(1,0), (0,1)])); c                     # optional - palp
             1-d cone of Rational polyhedral fan in 1-d lattice N
-            sage: f.preimage_cones(c)
+            sage: f.preimage_cones(c)                                           # optional - palp
             (1-d cone of Rational polyhedral fan in 2-d lattice N,
              2-d cone of Rational polyhedral fan in 2-d lattice N,
              2-d cone of Rational polyhedral fan in 2-d lattice N)
@@ -1674,8 +1676,8 @@ class FanMorphism(FreeModuleMorphism):
 
         Consider a projection of a del Pezzo surface onto the projective line::
 
-            sage: Sigma = toric_varieties.dP6().fan()
-            sage: Sigma.rays()
+            sage: Sigma = toric_varieties.dP6().fan()                           # optional - palp
+            sage: Sigma.rays()                                                  # optional - palp
             N( 0,  1),
             N(-1,  0),
             N(-1, -1),
@@ -1684,7 +1686,7 @@ class FanMorphism(FreeModuleMorphism):
             N( 1,  1)
             in 2-d lattice N
             sage: Sigma_p = toric_varieties.P1().fan()
-            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)
+            sage: phi = FanMorphism(matrix([[1], [-1]]), Sigma, Sigma_p)        # optional - palp
 
         Under this map, one pair of rays is mapped to the origin, one in the
         positive direction, and one in the negative one. Also three
@@ -1692,28 +1694,28 @@ class FanMorphism(FreeModuleMorphism):
         the negative one, so there are 5 preimage cones corresponding to either
         of the rays of the codomain fan ``Sigma_p``::
 
-            sage: len(phi.preimage_cones(Cone([(1,)])))
+            sage: len(phi.preimage_cones(Cone([(1,)])))                         # optional - palp
             5
 
         Yet only rays are primitive::
 
-            sage: phi.primitive_preimage_cones(Cone([(1,)]))
+            sage: phi.primitive_preimage_cones(Cone([(1,)]))                    # optional - palp
             (1-d cone of Rational polyhedral fan in 2-d lattice N,
              1-d cone of Rational polyhedral fan in 2-d lattice N)
 
         Since all primitive cones are mapped onto their images bijectively, we
         get a fibration::
 
-            sage: phi.is_fibration()
+            sage: phi.is_fibration()                                            # optional - palp
             True
 
         But since there are several primitive cones corresponding to the same
         cone of the codomain fan, this map is not a bundle, even though its
         index is 1::
 
-            sage: phi.is_bundle()
+            sage: phi.is_bundle()                                               # optional - palp
             False
-            sage: phi.index()
+            sage: phi.index()                                                   # optional - palp
             1
         """
         sigma_p = self._codomain_fan.embed(cone) # Necessary if used as a key
@@ -1767,10 +1769,10 @@ class FanMorphism(FreeModuleMorphism):
         coordinate planes"::
 
             sage: A2 = toric_varieties.A2()
-            sage: P3 = toric_varieties.P(3)
+            sage: P3 = toric_varieties.P(3)                                             # optional - palp
             sage: m = matrix([(2,0,0), (1,1,0)])
-            sage: phi = A2.hom(m, P3)
-            sage: phi.as_polynomial_map()
+            sage: phi = A2.hom(m, P3)                                                   # optional - palp
+            sage: phi.as_polynomial_map()                                               # optional - palp
             Scheme morphism:
               From: 2-d affine toric variety
               To:   3-d CPR-Fano toric variety covered by 4 affine patches
@@ -1779,19 +1781,18 @@ class FanMorphism(FreeModuleMorphism):
 
         Now we will work with the underlying fan morphism::
 
-            sage: phi = phi.fan_morphism()
-            sage: phi
+            sage: phi = phi.fan_morphism(); phi                                         # optional - palp
             Fan morphism defined by the matrix
             [2 0 0]
             [1 1 0]
             Domain fan: Rational polyhedral fan in 2-d lattice N
             Codomain fan: Rational polyhedral fan in 3-d lattice N
-            sage: phi.is_surjective(), phi.is_birational(), phi.is_injective()
+            sage: phi.is_surjective(), phi.is_birational(), phi.is_injective()          # optional - palp
             (False, False, False)
-            sage: phi_i, phi_b, phi_s = phi.factor()
-            sage: phi_s.is_surjective(), phi_b.is_birational(), phi_i.is_injective()
+            sage: phi_i, phi_b, phi_s = phi.factor()                                    # optional - palp
+            sage: phi_s.is_surjective(), phi_b.is_birational(), phi_i.is_injective()    # optional - palp
             (True, True, True)
-            sage: prod(phi.factor()) == phi
+            sage: prod(phi.factor()) == phi                                             # optional - palp
             True
 
         Double cover (surjective)::
@@ -1800,26 +1801,26 @@ class FanMorphism(FreeModuleMorphism):
             N(1, 0),
             N(0, 1)
             in 2-d lattice N
-            sage: phi_s
+            sage: phi_s                                                                 # optional - palp
             Fan morphism defined by the matrix
             [2 0]
             [1 1]
             Domain fan: Rational polyhedral fan in 2-d lattice N
             Codomain fan: Rational polyhedral fan in Sublattice <N(1, 0, 0), N(0, 1, 0)>
-            sage: phi_s.codomain_fan().rays()
+            sage: phi_s.codomain_fan().rays()                                           # optional - palp
             N(1, 0, 0),
             N(1, 1, 0)
             in Sublattice <N(1, 0, 0), N(0, 1, 0)>
 
         Blowup chart (birational)::
 
-            sage: phi_b
+            sage: phi_b                                                                 # optional - palp
             Fan morphism defined by the matrix
             [1 0]
             [0 1]
             Domain fan: Rational polyhedral fan in Sublattice <N(1, 0, 0), N(0, 1, 0)>
             Codomain fan: Rational polyhedral fan in Sublattice <N(1, 0, 0), N(0, 1, 0)>
-            sage: phi_b.codomain_fan().rays()
+            sage: phi_b.codomain_fan().rays()                                           # optional - palp
             N(-1, -1, 0),
             N( 0,  1, 0),
             N( 1,  0, 0)
@@ -1827,13 +1828,13 @@ class FanMorphism(FreeModuleMorphism):
 
         Coordinate plane inclusion (injective)::
 
-            sage: phi_i
+            sage: phi_i                                                                 # optional - palp
             Fan morphism defined by the matrix
             [1 0 0]
             [0 1 0]
             Domain fan: Rational polyhedral fan in Sublattice <N(1, 0, 0), N(0, 1, 0)>
             Codomain fan: Rational polyhedral fan in 3-d lattice N
-            sage: phi.codomain_fan().rays()
+            sage: phi.codomain_fan().rays()                                             # optional - palp
             N( 1,  0,  0),
             N( 0,  1,  0),
             N( 0,  0,  1),
@@ -1842,16 +1843,16 @@ class FanMorphism(FreeModuleMorphism):
 
         TESTS::
 
-            sage: phi_s.matrix() * phi_b.matrix() * phi_i.matrix() == m
+            sage: phi_s.matrix() * phi_b.matrix() * phi_i.matrix() == m                 # optional - palp
             True
 
-            sage: phi.domain_fan() is phi_s.domain_fan()
+            sage: phi.domain_fan() is phi_s.domain_fan()                                # optional - palp
             True
-            sage: phi_s.codomain_fan() is phi_b.domain_fan()
+            sage: phi_s.codomain_fan() is phi_b.domain_fan()                            # optional - palp
             True
-            sage: phi_b.codomain_fan() is phi_i.domain_fan()
+            sage: phi_b.codomain_fan() is phi_i.domain_fan()                            # optional - palp
             True
-            sage: phi_i.codomain_fan() is phi.codomain_fan()
+            sage: phi_i.codomain_fan() is phi.codomain_fan()                            # optional - palp
             True
 
             sage: trivialfan2 = Fan([],[],lattice=ToricLattice(2))
