@@ -24,13 +24,14 @@ from sage.rings.finite_rings.finite_field_constructor import FiniteField as GF
 from sage.misc.misc import cputime
 from cysignals.alarm import AlarmInterrupt, alarm, cancel_alarm
 
-from sage.interfaces.all import magma
+from sage.interfaces.magma import magma
 
 verbose = False
 
 timeout = 60
 
-def report(F, title, systems = ['sage', 'magma'], **kwds):
+
+def report(F, title, systems=['sage', 'magma'], **kwds):
     """
     Run benchmarks with default arguments for each function in the list F.
 
@@ -150,7 +151,8 @@ t := Cputime();
 K := Kernel(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -189,7 +191,8 @@ t := Cputime();
 K := CharacteristicPolynomial(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -228,7 +231,8 @@ t := Cputime();
 K := Rank(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -266,7 +270,8 @@ t := Cputime();
 K := Rank(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -306,7 +311,8 @@ t := Cputime();
 K := ElementaryDivisors(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -351,7 +357,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n,min,max,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))/times
     else:
@@ -397,7 +404,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n,min,max,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))/times
     else:
@@ -457,7 +465,8 @@ t := Cputime();
 d := Determinant(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -497,7 +506,8 @@ t := Cputime();
 d := Determinant(A);
 s := Cputime(t);
 """%(n,-num_bound, num_bound, den_bound)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -544,7 +554,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n,min,max,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))/times
     else:
@@ -620,7 +631,8 @@ t := Cputime();
 K := Kernel(A);
 s := Cputime(t);
 """%(n,p)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return magma.eval('s')
     else:
@@ -659,7 +671,8 @@ t := Cputime();
 K := CharacteristicPolynomial(A);
 s := Cputime(t);
 """%(n,p)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return magma.eval('s')
     else:
@@ -700,7 +713,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n,p,p,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return magma.eval('s')
     else:
@@ -746,7 +760,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n,p,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))/times
     else:
@@ -783,7 +798,8 @@ t := Cputime();
 K := Rank(A);
 s := Cputime(t);
 """%(n,p)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -819,7 +835,8 @@ t := Cputime();
 K := Rank(A);
 s := Cputime(t);
 """%(n,p)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -856,7 +873,8 @@ t := Cputime();
 d := Determinant(A);
 s := Cputime(t);
 """%(n,p)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -918,7 +936,8 @@ t := Cputime();
 K := EchelonForm(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -957,7 +976,8 @@ t := Cputime();
 K := A^(-1);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -1003,7 +1023,8 @@ for z in [1..%s] do
 end for;
 s := Cputime(t);
 """%(n, A.name(), times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))/times
     else:
@@ -1040,7 +1061,8 @@ d := Determinant(h);
 s := Cputime(tinit);
 delete h;
 """%n
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
 
@@ -1074,7 +1096,8 @@ d := h^(-1);
 s := Cputime(tinit);
 delete h;
 """%n
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
 
@@ -1117,7 +1140,8 @@ def MatrixVector_QQ(n=1000,h=100,system='sage',times=1):
             end for;
             s := Cputime(t);
         """%(n,h,times)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -1165,7 +1189,8 @@ t := Cputime();
 K := Kernel(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
@@ -1205,7 +1230,8 @@ t := Cputime();
 K := Kernel(A);
 s := Cputime(t);
 """%(n,min,max)
-        if verbose: print(code)
+        if verbose:
+            print(code)
         magma.eval(code)
         return float(magma.eval('s'))
     else:
