@@ -83,10 +83,10 @@ def canonical_parameters(group, level, weight, base_ring):
         sage: v = canonical_parameters(5, 5, int(7), ZZ); v
         (5, Congruence Subgroup Gamma0(5), 7, Integer Ring)
         sage: type(v[0]), type(v[1]), type(v[2]), type(v[3])
-        (<type 'sage.rings.integer.Integer'>,
+        (<class 'sage.rings.integer.Integer'>,
          <class 'sage.modular.arithgroup.congroup_gamma0.Gamma0_class_with_category'>,
-         <type 'sage.rings.integer.Integer'>,
-         <type 'sage.rings.integer_ring.IntegerRing_class'>)
+         <class 'sage.rings.integer.Integer'>,
+         <class 'sage.rings.integer_ring.IntegerRing_class'>)
         sage: canonical_parameters( 5, 7, 7, ZZ )
         Traceback (most recent call last):
         ...
@@ -313,10 +313,10 @@ def ModularForms(group  = 1,
         raise ValueError("eis_only parameter only valid in weight 1")
 
     if use_cache and key in _cache:
-         M = _cache[key]()
-         if not (M is None):
-             M.set_precision(prec)
-             return M
+        M = _cache[key]()
+        if not (M is None):
+            M.set_precision(prec)
+            return M
 
     (level, group, weight, base_ring, eis_only) = key
 
