@@ -61,6 +61,7 @@ from sage.structure.element import is_Matrix
 from sage.arith.misc import gcd
 from sage.combinat.root_system.cartan_matrix import CartanMatrix
 from sage.misc.cachefunc import cached_method
+from sage.quadratic_forms.all import QuadraticForm
 
 ###############################################################################
 #
@@ -549,8 +550,8 @@ def IntegralLatticeGluing(Lattices, glue, return_embeddings=False):
         sage: [L, phi] = IntegralLatticeGluing([L1, L2], [[f1, g1], [f2, 2 * g2]], True)
         sage: phi[0]
         Free module morphism defined by the matrix
-        [ 2  2 -1 -2]
-        [ 0  2  0 -1]
+        [ 2  2 -2 -1]
+        [ 0  2 -1  0]
         Domain: Lattice of degree 4 and rank 2 over Integer Ring
         Basis matrix:
         [1 1 0 0]
@@ -563,7 +564,7 @@ def IntegralLatticeGluing(Lattices, glue, return_embeddings=False):
         Codomain: Lattice of degree 10 and rank 4 over Integer Ring
         Basis matrix:
         [ 1/2    0 -1/2    0    0  1/2    0    0  1/2  1/2]
-        [   0  1/2  1/2    0    0    0    0    0  1/2  1/2]
+        [   0  1/2  1/2    0    0  1/2    0    0    0    0]
         [   0    0    0    0    0    1    0    0    0    0]
         [   0    0    0    0    0    0    0    0    1    1]
         Inner product matrix:
@@ -784,7 +785,7 @@ class FreeQuadraticModule_integer_symmetric(FreeQuadraticModule_submodule_with_b
             Finite quadratic module over Integer Ring with invariants (2, 10)
             Gram matrix of the quadratic form with values in Q/2Z:
             [  1 1/2]
-            [1/2 9/5]
+            [1/2 1/5]
             sage: L.discriminant_group(2)
             Finite quadratic module over Integer Ring with invariants (2, 2)
             Gram matrix of the quadratic form with values in Q/2Z:
@@ -793,7 +794,7 @@ class FreeQuadraticModule_integer_symmetric(FreeQuadraticModule_submodule_with_b
             sage: L.discriminant_group(5)
             Finite quadratic module over Integer Ring with invariants (5,)
             Gram matrix of the quadratic form with values in Q/2Z:
-            [6/5]
+            [4/5]
 
         TESTS::
 
@@ -1049,8 +1050,8 @@ class FreeQuadraticModule_integer_symmetric(FreeQuadraticModule_submodule_with_b
         # https://arxiv.org/abs/1208.2481
         # and trac:11940
         if not self.is_even() and (p is None or p==2):
-            raise ValueError("This lattice must be even to admit an even overlattice")
-        from sage.rings.all import GF
+            raise ValueError("this lattice must be even to admit an even overlattice")
+        from sage.rings.finite_rings.finite_field_constructor import GF
         L = self
         if p is None:
             P = ZZ(self.determinant()).prime_factors()
@@ -1374,6 +1375,146 @@ class FreeQuadraticModule_integer_symmetric(FreeQuadraticModule_submodule_with_b
                                                          basis=basis_matrix,
                                                          inner_product_matrix=ambient.inner_product_matrix())
 
+    @cached_method
+    def quadratic_form(self):
+        r"""
+        Return the quadratic form given by `q(x)=(x,x)`.
+
+        EXAMPLES::
+
+            sage: L = IntegralLattice("A2")
+            sage: q = L.quadratic_form()
+            sage: q
+            Quadratic form in 2 variables over Integer Ring with coefficients:
+            [ 2 -2 ]
+            [ * 2 ]
+        """
+        return QuadraticForm(2 * self.gram_matrix())
+
+    @cached_method
+    def minimum(self):
+        r"""
+        Return the minimum of this lattice.
+
+        .. MATH::
+
+            \min\{x^2 | x \in L\setminus \{0\}\}
+
+        EXAMPLES::
+
+            sage: L = IntegralLattice('A2')
+            sage: L.minimum()
+            2
+            sage: L.twist(-1).minimum()
+            -Infinity
+        """
+        p, n = self.signature_pair()
+        if self.rank() == 0:
+            raise ValueError("the empty set does not have a minimum")
+        if n != 0:
+            from sage.rings.infinity import MinusInfinity
+            return MinusInfinity()
+        mpari = self.gram_matrix().__pari__().qfminim(None, 0)[1]
+        return mpari
+
+    @cached_method
+    def maximum(self):
+        r"""
+        Return the maximum of this lattice.
+
+        .. MATH::
+
+            \max\{x^2 | x \in L\setminus \{0\}\}
+
+        EXAMPLES::
+
+            sage: L = IntegralLattice('A2')
+            sage: L.maximum()
+            +Infinity
+            sage: L.twist(-1).maximum()
+            -2
+        """
+        if self.rank() == 0:
+            raise ValueError("the empty set does not have a maximum")
+        p, n = self.signature_pair()
+        if p != 0:
+            from sage.rings.infinity import PlusInfinity
+            return PlusInfinity()
+        mpari = (-self.gram_matrix()).__pari__().qfminim(None, 0)[1]
+        return -mpari
+
+    min = minimum
+    max = maximum
+
+    def LLL(self):
+        r"""
+        Return this lattice with an LLL reduced basis.
+
+        EXAMPLES::
+
+            sage: L = IntegralLattice('A2')
+            sage: L.lll() == L
+            True
+            sage: G = matrix(ZZ,3,[0,1,0, 1,0,0, 0,0,7])
+            sage: V = matrix(ZZ,3,[-14,-15,-15, -4,1,16, -5,-5,-4])
+            sage: L = IntegralLattice(V * G * V.T)
+            sage: L.lll().gram_matrix()
+            [0 0 1]
+            [0 7 0]
+            [1 0 0]
+        """
+        p, n = self.signature_pair()
+        if p * n != 0:
+            from sage.env import SAGE_EXTCODE
+            from sage.interfaces.gp import gp
+            from sage.libs.pari import pari
+            m = self.gram_matrix().__pari__()
+            gp.read(SAGE_EXTCODE + "/pari/simon/qfsolve.gp")
+            m = gp.eval('qflllgram_indefgoon(%s)'%m)
+            # convert the output string to sage
+            G, U = pari(m).sage()
+            U = U.T
+        else:
+            e = 1
+            if n != 0:
+                e = -1
+            U = (e * self.gram_matrix().change_ring(ZZ)).LLL_gram().T
+        return self.sublattice(U * self.basis_matrix())
+
+    lll = LLL
+
+    def short_vectors(self, n, **kwargs):
+        r"""
+        Return the short vectors of length `< n`.
+
+        INPUT:
+
+        - ``n`` -- an integer
+        - further key word arguments are passed on to
+          :meth:`sage.quadratic_forms.short_vector_list_up_to_length`.
+
+        OUTPUT:
+
+        - a list `L` where ``L[k]`` is the list of vectors of lengths `k`
+
+        EXAMPLES::
+
+            sage: A2 = IntegralLattice('A2')
+            sage: A2.short_vectors(3)
+            [[(0, 0)], [], [(1, 1), (-1, -1), (0, 1), (0, -1), (1, 0), (-1, 0)]]
+            sage: A2.short_vectors(3,up_to_sign_flag=True)
+            [[(0, 0)], [], [(1, 1), (0, 1), (1, 0)]]
+        """
+        p, m = self.signature_pair()
+        if p * m != 0:
+            raise NotImplementedError("the lattice has to be positive definite")
+        e = 2
+        if m != 0:
+            e = -2
+        q = QuadraticForm(e * self.gram_matrix())
+        short = q.short_vector_list_up_to_length(n, *kwargs)
+        return [[self(v * self.basis_matrix()) for v in L] for L in short]
+
     def twist(self, s, discard_basis=False):
         r"""
         Return the lattice with inner product matrix scaled by ``s``.
@@ -1463,8 +1604,8 @@ def local_modification(M, G, p, check=True):
         sage: local_modification(M, L.gram_matrix(), 2)
         Lattice of degree 4 and rank 4 over Integer Ring
         Basis matrix:
-        [1/3   0 1/3 2/3]
-        [  0 1/3 1/3 2/3]
+        [1/3   0 2/3 2/3]
+        [  0 1/3   0 2/3]
         [  0   0   1   0]
         [  0   0   0   1]
         Inner product matrix:
