@@ -1,5 +1,29 @@
 """
 Rings
+
+Tests for deprecations of imports in global namespace from :trac:`33602`::
+
+    sage: PowerSeries
+    doctest:warning...:
+    DeprecationWarning:
+    Importing PowerSeries from here is deprecated;
+    please use "from sage.rings.power_series_ring_element import PowerSeries" instead.
+    See https://trac.sagemath.org/33602 for details.
+    ...
+    sage: PuiseuxSeries
+    doctest:warning...:
+    DeprecationWarning:
+    Importing PuiseuxSeries from here is deprecated;
+    please use "from sage.rings.puiseux_series_ring_element import PuiseuxSeries" instead.
+    See https://trac.sagemath.org/33602 for details.
+    ...
+    sage: LaurentSeries
+    doctest:warning...:
+    DeprecationWarning:
+    Importing LaurentSeries from here is deprecated;
+    please use "from sage.rings.laurent_series_ring_element import LaurentSeries" instead.
+    See https://trac.sagemath.org/33602 for details.
+    ...
 """
 # ****************************************************************************
 #       Copyright (C) 2005 William Stein <wstein@gmail.com>
@@ -12,7 +36,7 @@ Rings
 # ****************************************************************************
 from sage.misc.lazy_import import lazy_import
 
-from .all__sage_categories import *
+from .all__sagemath_categories import *
 
 # Ring base classes
 from .ring import (Ring, Field, CommutativeRing, IntegralDomain,
@@ -115,21 +139,22 @@ lazy_import("sage.rings.imaginary_unit", "I")
 
 # Power series rings
 from .power_series_ring import PowerSeriesRing
-from .power_series_ring_element import PowerSeries
+lazy_import('sage.rings.power_series_ring_element', 'PowerSeries', deprecation=33602)
 
 # Laurent series ring in one variable
 from .laurent_series_ring import LaurentSeriesRing
-from .laurent_series_ring_element import LaurentSeries
+lazy_import('sage.rings.laurent_series_ring_element', 'LaurentSeries', deprecation=33602)
 
 # Lazy Laurent series ring
-lazy_import('sage.rings.lazy_laurent_series_ring', 'LazyLaurentSeriesRing')
+lazy_import('sage.rings.lazy_series_ring', ['LazyLaurentSeriesRing', 'LazyTaylorSeriesRing',
+                                            'LazySymmetricFunctions', 'LazyDirichletSeriesRing'])
 
 # Tate algebras
 from .tate_algebra import TateAlgebra
 
 # Puiseux series ring
 from .puiseux_series_ring import PuiseuxSeriesRing
-from .puiseux_series_ring_element import PuiseuxSeries
+lazy_import('sage.rings.puiseux_series_ring_element', 'PuiseuxSeries', deprecation=33602)
 
 # Pseudo-ring of PARI objects.
 from .pari_ring import PariRing, Pari
@@ -151,8 +176,8 @@ from .bernoulli_mod_p import bernoulli_mod_p, bernoulli_mod_p_single
 
 from .monomials import monomials
 
-CC = ComplexField()
-CIF = ComplexIntervalField()
+from .cc import CC
+from .cif import CIF
 
 # invariant theory
 from .invariants.all import *
