@@ -66,7 +66,7 @@ from sage.categories.enumerated_sets import EnumeratedSets
 from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
 from sage.misc.function_mangling import ArgumentFixer
 from sage.misc.lazy_list import lazy_list
-from sage.docs.instancedoc import instancedoc
+from sage.misc.instancedoc import instancedoc
 
 
 class EnumeratedSetFromIterator(Parent):
@@ -389,7 +389,7 @@ class EnumeratedSetFromIterator(Parent):
         """
         if hasattr(self, '_cache'):
             return self._cache[i]
-        return super(EnumeratedSetFromIterator,self).unrank(i)
+        return super().unrank(i)
 
     def _element_constructor_(self, el):
         """
@@ -399,10 +399,8 @@ class EnumeratedSetFromIterator(Parent):
 
             sage: from sage.sets.set_from_iterator import EnumeratedSetFromIterator
             sage: S = EnumeratedSetFromIterator(range, args=(1,4))
-            sage: S(1)  # py2
-            1
 
-            sage: S(1)  # py3
+            sage: S(1)
             doctest:...: UserWarning: Testing equality of infinite sets which will not end in case of equality
             1
             sage: S(0)  # indirect doctest
@@ -444,7 +442,7 @@ class EnumeratedSetFromIterator(Parent):
 
 #TODO: move it in sage.misc ?
 @instancedoc
-class Decorator(object):
+class Decorator():
     r"""
     Abstract class that manage documentation and sources of the wrapped object.
 
@@ -465,23 +463,16 @@ class Decorator(object):
             ...
                Calls the PARI "isprime" function.
         """
-        from sage.misc.sageinspect import sage_getsourcelines, sage_getfile, _extract_embedded_position
+        # Duplicates sage.misc.cachefunc.CachedFunction._instancedoc_
+        from sage.misc.sageinspect import sage_getsourcelines, sage_getfile_relative, _extract_embedded_position
         f = self.f
         doc = f.__doc__ or ''
         if _extract_embedded_position(doc) is None:
             try:
                 sourcelines = sage_getsourcelines(f)
-                from sage.env import SAGE_LIB, SAGE_SRC
-                filename = sage_getfile(f)
-                # The following is a heuristics to get
-                # the file name of the cached function
-                # or method
-                if filename.startswith(SAGE_SRC):
-                    filename = filename[len(SAGE_SRC):]
-                elif filename.startswith(SAGE_LIB):
-                    filename = filename[len(SAGE_LIB):]
-                file_info = "File: %s (starting at line %d)\n"%(filename,sourcelines[1])
-                doc = file_info+doc
+                filename = sage_getfile_relative(f)
+                file_info = "File: %s (starting at line %d)\n" % (filename, sourcelines[1])
+                doc = file_info + doc
             except IOError:
                 pass
         return doc
@@ -750,11 +741,7 @@ class EnumeratedSetFromIterator_method_caller(Decorator):
 
         But not the enumerated set::
 
-            sage: loads(dumps(d.f())) # py2
-            Traceback (most recent call last):
-            ...
-            PicklingError: Can't pickle <... 'function'>: attribute lookup __builtin__.function failed
-            sage: loads(dumps(d.f())) # py3
+            sage: loads(dumps(d.f()))
             Traceback (most recent call last):
             ...
             _pickle.PicklingError: Can't pickle <function DummyExampleForPicklingTest.f at ...>: it's not the same object as sage.sets.set_from_iterator.DummyExampleForPicklingTest.f
@@ -842,7 +829,7 @@ class EnumeratedSetFromIterator_method_caller(Decorator):
                 self.name,
                 **self.options)
 
-class EnumeratedSetFromIterator_method_decorator(object):
+class EnumeratedSetFromIterator_method_decorator():
     r"""
     Decorator for enumerated set built from a method.
 
